//	*********************************** //
// 	*********************************** //
// 	Empirical analysis for				//
// 	"Inflation, Debt, and Default," 	//
// 	by sewon Hur, Illenin Kondo,		//
//	and Fabrizio Perri					//					
//	Sources: 							//
//		>> OECD: GDP, consumption, debt	//
//		>> IFS: interest rates			//
//		>> Oxford: debt					//
//		>> Haver: interest rates, CDS	//
//										//
// 	Last modified: Sep 28, 2018			//
// 	*********************************** //

clear all
set more off

global span	= 10	// number of years in the window

// 	***********************************	//
//	LOAD DATA							//
// 	***********************************	//
use hkp_rawdata, clear
cap mkdir auxiliary
cap mkdir figures
cap mkdir tables

// set up panel
tsset ccode yr_qtr

// codes for loops
quietly levelsof ccode, local(ccodes)
quietly levelsof isocode, local(isocodes)

// 	*********************************** //
//	COUNTRY GROUPS						//
// 	*********************************** //

// 	Euro Area
quietly gen ea	= 	///
	  (isocode == "AUT")	| (isocode == "BEL")	| (isocode == "FIN")	///
  	| (isocode == "FRA") 	| (isocode == "DEU")	| (isocode == "GRC")	///
	| (isocode == "IRL")	| (isocode == "ITA") 	| (isocode == "LUX")	///
	| (isocode == "NLD") 	| (isocode == "PRT") 	| (isocode == "ESP")  

// 	***********************************	//
//	CONSTRUCTED VARIABLES				//
// 	***********************************	//

// 	Inflation
gen deflator	= ngdp/rgdp
label var deflator "GDP Deflator"
gen log_deflator	= log(deflator)
label var log_deflator "Log Deflator"
gen infl	= 100*(log_deflator-l.log_deflator)
label var infl "Inflation in pct"

// 	log RGDP and log RCONS
gen lrgdp	= log(rgdp)
label var lrgdp "Log Real GDP"

gen lngdp	= log(ngdp)
label var lngdp "Log Nominal GDP"

// 	Real consumption
gen rcons	= rpcons + rgcons
label var rcons "Real Total Consumption"

gen lrcons	= log(rcons)
label var lrcons "Log Real Total Consumption"

gen dlcons 	= 100*d.lrcons
label var dlcons "Log change of Real Total Consumption (percent)"

// 	Government consumption
gen gcons_gdp = .
replace gcons_gdp = ngcons/ngdp*100
label var gcons_gdp "Government consumption (percent of GDP)"

// 	Default probability (assume uniform recovery rate of 50 percent)
// gen defprob = .
// replace defprob = min(100,nyieldcds/(1-0.5)) if nyieldcds!=.
// label var defprob "CDS implied default probability"

// Composition of Debt
gen debt_dom_cred	= 100*govdebt_dom_creditor	///
	/(govdebt_dom_creditor+govdebt_ext_creditor)
label var debt_dom_cred "Percent of government debt held by domestic creditors"
	
// 	*********************************** //
// 	HP FILTER 							//
// 	*********************************** //

// 	HP filtered log consumption 
// ssc install hprescott
by ccode: hprescott lrcons, stub(hp)
drop hp_lrcons_sm_*
egen double lcons_f = rowtotal(hp_lrcons_*)
replace lcons_f = 100*lcons_f
drop hp_lrcons_*
replace lcons_f = . if lcons_f==.

// 	HP filtered inflation
by ccode: hprescott log_deflator, stub(hp)
drop hp_log_deflator_sm_*
egen double infl_f = rowtotal(hp_log_deflator_*)
replace infl_f = 100*infl_f
drop hp_log_deflator_*
replace infl_f = . if deflator==.

//	keep 1984.q4 to 2016.q1
drop if yr_qtr<99 |yr_qtr>224

save auxiliary\hkp_main, replace


//	*********************************** //
//	VAR (HP RESIDUALS)               	//
//	*********************************** //
 
// 	VAR (infl_f,lcons_f) - For calibration purposes, estimate VAR on HP residuals

save auxiliary\hkp_vars, replace

quietly foreach cn of local isocodes {
	
	var infl_f lcons_f if isocode =="`cn'", noconstant lags(1)
	
	if _rc == 0 {
	
		matlist e(Sigma)
		
		keep isocode 
		keep if isocode == "`cn'"

		matrix coefficients = e(b) // store the 1-by-4 vector of coefficients
		matrix varcov = e(Sigma) // store the 2-by-2 variance-covariance matrix
		
		// convert to 1-by-4 vector
		mata: varcov=st_matrix("varcov")
		mata: varcov=colshape(varcov,4)
		mata: st_matrix("varcov", varcov)

		svmat coefficients
		svmat varcov

		rename coefficients1 rho_pi_pi
		rename coefficients2 rho_c_pi
		rename coefficients3 rho_pi_c
		rename coefficients4 rho_c_c
		rename varcov1 var_pi
		rename varcov2 cov_pi_c
		rename varcov4 var_c
			
		keep isocode rho_pi_pi rho_c_pi rho_pi_c rho_c_c var_pi cov_pi_c var_c
		keep if var_c~=.

		merge 1:m isocode using auxiliary\hkp_vars
		drop _merge	
		
		save auxiliary\hkp_vars, replace 
		
	}
} 

//	summarize VAR estimates
use auxiliary\hkp_vars, clear
duplicates drop isocode, force
gen std_pi = sqrt(var_pi)
gen std_c = sqrt(var_c)
keep isocode rho_pi_pi rho_c_pi rho_pi_c rho_c_c var_pi std_pi var_c std_c cov_pi_c
order isocode rho_pi_pi rho_c_pi rho_pi_c rho_c_c var_pi std_pi var_c std_c cov_pi_c
sum rho* var* cov*, detail
export excel tables\table_10, firstrow(variables) sheetmodify


// 	*********************************** //
// 	VAR, EXPECTED INFLATION  			//
// 	*********************************** //

// VAR (infl,dlcons) - For empirical analysis, estimate VAR on log changes

use auxiliary\hkp_main, clear

// expected inflation
gen exp_infl 	= .

// residual inflation, residual consumption
gen res_infl 	= .  
gen res_dlcons 	= .

quietly foreach cn of local isocodes {
	
	var infl dlcons if isocode =="`cn'", lags(1)
	
	if _rc == 0 {
	
		matlist e(Sigma)
		
		predict einfl if e(sample), xb eq(#1)   
		predict ehat_infl if e(sample), res eq(#1) 
		predict ehat_lcons if e(sample), res eq(#2)

		replace exp_infl	= einfl if e(sample)
		replace res_infl 	= ehat_infl if e(sample)
		replace res_dlcons 	= ehat_lcons if e(sample)
		drop einfl ehat_infl ehat_lcons 
	}
} 


// 	*********************************** //
// 	REAL YIELDS & SPREADS				//
// 	*********************************** //

keep if year>=1985 & year<=2015

// real yield
gen ryield = nyieldifs-(4*F.exp_infl) // nominal rate minus expected inflation

// nominal yield in Germany
bysort yr_qtr: egen nyield_DEU = total(nyieldifs*(isocode == "DEU")), missing
xtset ccode yr_qtr

// Spread wrt to Germany
gen spr_nyield = nyieldifs - nyield_DEU  if ea & (isocode ~= "DEU")

drop *_DEU

// 	***********************************	//
//  UNCOND'L STATS 						//
// 	***********************************	//

// initialization for unconditional stats
local uncvars "infl dlcons lcons_f"
quietly foreach uvar of local uncvars {
	gen av_unc_`uvar' 	= .
	gen var_unc_`uvar' 	= .
	gen sd_unc_`uvar' 	= .
}

quietly foreach acode of local ccodes {		
	foreach uvar of local uncvars {
		cap  summarize `uvar' if ccode == `acode'
		if _rc == 0 {
			replace av_unc_`uvar' = r(mean)	///
				if ccode == `acode'
			replace var_unc_`uvar' = r(Var)	///
				if ccode == `acode'
			replace sd_unc_`uvar' = r(sd) 	///
				if ccode == `acode'
		}
	} 
}

save auxiliary\hkp_main, replace

// 	***********************************	//
//  SUMMARY STATISTICS 					//
// 	***********************************	//
use auxiliary\hkp_main, clear

// 	Compute calibration targets (Table 3)

// 	Govt consumption shares
eststo clear
eststo govtcons: estpost tabstat gcons_gdp

// 	Govt debt maturity (Duration Macaulay)
eststo debtmaturity: estpost tabstat dur_mac_domestic if year>=1999

//	Safe-Risky transition probabilities
keep if spr_nyield~=.
gen safe_regime=0
gen risky_regime=0

replace safe_regime=1 if spr_nyield<2.0
replace risky_regime=1 if spr_nyield>=2.0

eststo safe: reg f.safe_regime safe_regime if safe_regime==1 & year>=1999, noconstant
eststo risky: reg f.risky_regime risky_regime if risky_regime==1 & year>=1999, noconstant

esttab using tables\table_3.csv, cells("b mean") ///
	title("TABLE 3: Calibration") ///
	addnotes("(1) Average government consumption (percent GDP 1985-2015)" ///
	"(2) Duration Macaulay for domestic debt (years 1999-2010)" ///
	"(3) Persistence of safe regime (Eurozone 1999-2015)"  ///
	"(4) Persistence of risky regime (Eurozone 1999-2015)"  ) ///
	noobs replace
eststo clear

use auxiliary\hkp_main, clear

//	Share of public debt held by domestic creditors (Footnote 2)
keep if year==2015 & quarter==4
keep country isocode debt_dom_cred year quarter
export excel tables\footnote_2, firstrow(variables) replace

//	Default probabilties (Footnote 3)
// use auxiliary\hkp_main, clear
// keep if year==2011 & quarter==4
// keep country isocode defprob yr_qtr
// export excel tables\footnote_3, firstrow(variables) replace

// 	***********************************	//
//  STATS BY WINDOWS 					//
// 	***********************************	//

use auxiliary\hkp_main, clear
xtset ccode yr_qtr
quietly levelsof yr_qtr, local(periods)

//	Configure windows
local wdw	= 4*${span} - 1			
local minsz	= floor(`wdw'*0.75) + 1	// minimum size for valid windows (30 quarters)

// initialization
local meanvars "res_infl res_dlcons infl dlcons lcons_f ryield spr_nyield"

quietly foreach mvar of local meanvars {
	gen av_`mvar' 		= .
	gen var_`mvar' 		= .
	gen sd_`mvar' 		= .
}

// initialization of covariance and correlations
quietly gen cor_res_infl_dlcons		= .
quietly gen cov_res_infl_dlcons		= .

// country-window statistics
// BEGIN - period loop
quietly foreach qtr of local periods {

	// create window of `span' years (= `wdw' qtrs)
	gen _window = yr_qtr >= `qtr' & yr_qtr <= `qtr' + `wdw' 
	
	// BEGIN - country loop
	foreach acode of local ccodes {		
		
		// BEGIN - valid country-window
		cap sum yr_qtr if _window & ccode == `acode'
		if r(N) > `minsz' & _rc == 0 {

			// BEGIN - Levels - consumption, inflation, debt, yields, spreads
			foreach mvar of local meanvars {
				cap summarize `mvar' if _window & ccode == `acode'
				if r(N) > `minsz' & _rc == 0 { 
					replace av_`mvar' 	= r(mean) 	///
						if yr_qtr == `qtr' & ccode == `acode'
					replace var_`mvar' 	= r(Var)	///
						if yr_qtr == `qtr' & ccode == `acode'
					replace sd_`mvar' 	= r(sd)  	///
						if yr_qtr == `qtr' & ccode == `acode'
				}
			} 
			// END - Levels - consumption, inflation, debt
			
			// BEGIN  - Residuals - Inflation, consumption growth
			// Correlations
			cap corr res_infl res_dlcons if _window & ccode == `acode'
			if r(N) > `minsz' & _rc == 0 { 
				replace cor_res_infl_dlcons = r(rho)	///
					if yr_qtr == `qtr' & ccode == `acode'
			}
		
			// Covariances
			cap corr res_infl res_dlcons if _window & ccode == `acode', cov
			if r(N) > `minsz' & _rc == 0 {
				replace cov_res_infl_dlcons = r(cov_12)	///
					if yr_qtr == `qtr' & ccode == `acode'				
			}
			// END - Residuals - Inflation, consumption growth, yields, spreads
		
		} // END - valid country-window

	} // END - country loop
	
	drop _window
} // END - period loop

save auxiliary\hkp_main, replace

//	***********************************	//
// 	Main Figures and Tables				//
// 	*********************************** //

// 	***********************************	//
//	LOAD DATA							//
// 	*********************************** //

use auxiliary\hkp_main, clear

// 	Time and country dummies
quietly tabulate yr_qtr, gen(tdum)
quietly tabulate ccode, gen(cdum)

// 	Regression controls
global avgvars	"av_res_infl av_res_dlcons"
global volvars	"var_res_infl var_res_dlcons"

// 	Good/bad times based on residual consumption growth
// quietly reg av_dlcons tdum* cdum* 
// predict ehat_av_dlcons if e(sample), res
//
// cap drop goodwindow badwindow
// gen goodwindow = 0 if ehat_av_dlcons~=.
// gen badwindow = 0 if ehat_av_dlcons~=.
//
// replace goodwindow = 1 if ehat_av_dlcons>=0.0 & ehat_av_dlcons~=.
// replace badwindow = 1 if ehat_av_dlcons<0.0 & ehat_av_dlcons~=.
//
// gen int_cov_goodwin = cov_res_infl_dlcons*goodwindow
// gen int_cor_goodwin = cor_res_infl_dlcons*goodwindow
// gen int_cov_badwin = cov_res_infl_dlcons*badwindow
// gen int_cor_badwin = cor_res_infl_dlcons*badwindow

// 	Good/bad times based on credit ratings
// cap drop goodwindow_cr badwindow_cr
// gen goodwindow_cr = 0 if av_credit_rating~=.
// gen badwindow_cr = 0 if av_credit_rating~=.
//
// replace goodwindow_cr = 1 if av_credit_rating>=16.92 & av_credit_rating~=.
// replace badwindow_cr = 1 if av_credit_rating<16.92 & av_credit_rating~=.
//
// gen int_cov_goodwin_cr = cov_res_infl_dlcons*goodwindow_cr
// gen int_cor_goodwin_cr = cor_res_infl_dlcons*goodwindow_cr
// gen int_cov_badwin_cr = cov_res_infl_dlcons*badwindow_cr
// gen int_cor_badwin_cr = cor_res_infl_dlcons*badwindow_cr

// 	***********************************	//
//	GRAPH COMOVEMENT DATA				//
// 	*********************************** //

tsline cor_res_infl_dlcons, 	///
	by(ccode, cols(7) graphregion(fcolor(white))) 	///
	xlabel(#3,angle(45)) ylabel(-0.5 0 0.5) subtitle(,bcolor(white)) ///
	tlabel(1990q1(40)2010q4) ytitle("correlation") xtitle("")
graph save figures\figure_2.gph, asis replace
graph export figures\figure_2.pdf, as(pdf) replace

// 	***********************************	//
//	REGRESSION - YIELDS - MAIN	(ALL)	//
// 	*********************************** //

eststo clear
di  _newline(1) " REAL YIELD ON GOVT DEBT "
eststo cov1: quietly reg av_ryield cov_res_infl_dlcons 				///
	tdum* cdum*, vce(cluster ccode)
// eststo cov2: quietly reg av_ryield cov_res_infl_dlcons 				///
// 	l.av_govdebt $avgvars tdum* cdum*, vce(cluster ccode)
// eststo cov3: quietly reg av_ryield cov_res_infl_dlcons 				///
// 	l.av_govdebt $avgvars $volvars tdum* cdum*, vce(cluster ccode)
// eststo cor1: quietly reg av_ryield cor_res_infl_dlcons 				///
// 	l.av_govdebt $avgvars $volvars tdum* cdum*, vce(cluster ccode)
	
cap drop yield_sample 
cap: gen yield_sample = e(sample)
levelsof isocode if yield_sample, local(ccodes) sep(, ) clean

esttab, title("Table 1: Inflation consumption growth covariance and real interest rates") ///
	drop (*tdum* *cdum* _cons) b(a3) ar2 se noconstant compress star(* 0.10 ** 0.05 *** 0.01) ///
	addnotes("All regressions include time and country fixed effects. Standard errors clustered by country." "Countries: `ccodes'") 
esttab using tables\table_1.csv, title("Inflation consumption growth covariance and real interest rates")	///
	drop (*tdum* *cdum* _cons) b(a3) ar2 se noconstant compress star(* 0.10 ** 0.05 *** 0.01) ///
	addnotes("All regressions include country and time fixed effects. Standard errors clustered by country." "Countries: `ccodes'") ///
	obslast replace
eststo clear

// di  _newline(1) " REAL YIELD ON GOVT DEBT "
// eststo cov1: quietly reg av_ryieldifs cov_res_infl_dlcons 				///
// 	l.av_govdebt $avgvars $volvars tdum* cdum*, vce(cluster ccode)
// eststo cov2: quietly reg av_ryieldifs goodwindow int_cov_goodwin int_cov_badwin	///
// 	l.av_govdebt $avgvars $volvars tdum* cdum*, vce(cluster ccode)	
// eststo cov3: quietly reg av_ryieldifs goodwindow_cr int_cov_goodwin_cr int_cov_badwin_cr	///
// 	l.av_govdebt $avgvars $volvars tdum* cdum*, vce(cluster ccode)
//	
// cap drop yield_sample 
// cap: gen yield_sample = e(sample)
// levelsof isocode if yield_sample, local(ccodes) sep(, ) clean
//
// esttab, title("Table 2: Inflation procyclicality discount in good times") ///
// 	drop (*tdum* *cdum* av_res* var_* L.av_* _cons) b(a3) ar2 se noconstant compress star(* 0.10 ** 0.05 *** 0.01) ///
// 	addnotes("All regressions include time and country fixed effects. Standard errors clustered by country." "Countries: `ccodes'") 
// esttab using tables\table_2.csv, title("Inflation procyclicality discount in good times")	///
// 	drop (*tdum* *cdum* av_res* var_* L.av_* _cons) b(a3) ar2 se noconstant compress star(* 0.10 ** 0.05 *** 0.01) ///
// 	addnotes("All regressions include country and time fixed effects. Standard errors clustered by country." "Countries: `ccodes'") ///
// 	obslast replace
// eststo clear

// 	***********************************	//
//	REGRESSION - DIFFERENT MATURITIES	//
// 	*********************************** //

// eststo clear
// di  _newline(1) " REAL YIELD ON GOVT DEBT "
// eststo ifs: quietly reg av_ryieldifs cov_res_infl_dlcons 				///
// 	$avgvars $volvars l.av_govdebt tdum* cdum*, vce(cluster ccode)
// eststo ten: quietly reg av_ryield10y cov_res_infl_dlcons 				///
// 	$avgvars $volvars l.av_govdebt tdum* cdum*, vce(cluster ccode)
// eststo fiv: quietly reg av_ryield5y cov_res_infl_dlcons 				///
// 	$avgvars $volvars l.av_govdebt tdum* cdum*, vce(cluster ccode)
// eststo short: quietly reg av_ryield3m cov_res_infl_dlcons 				///
// 	$avgvars $volvars l.av_govdebt tdum* cdum*, vce(cluster ccode)
//	
// cap drop yield_sample 
// cap: gen yield_sample = e(sample)
// levelsof isocode if yield_sample, local(ccodes) sep(, ) clean
//
// esttab, title("Table 8: Sensitivity to yield maturities") ///
// 	drop (*tdum* *cdum* av_res* var_* L.av_* _cons) b(a3) ar2 se noconstant compress star(* 0.10 ** 0.05 *** 0.01) ///
// 	addnotes("All regressions include time and country fixed effects. Standard errors clustered by country." "Countries: `ccodes'") 
// esttab using tables\table_8.csv, title("Table 8: Sensitivity to yield maturities")	///
// 	drop (*tdum* *cdum* av_res* var_* L.av_* _cons) b(a3) ar2 se noconstant compress star(* 0.10 ** 0.05 *** 0.01) ///
// 	addnotes("All regressions include country and time fixed effects. Standard errors clustered by country." "Countries: `ccodes'") ///
// 	obslast replace
// eststo clear

// 	***********************************	//
//	REGRESSION - YIELDS - ALT DEBT MEASURES		//
// 	*********************************** //
// eststo clear
// di  _newline(1) " REAL YIELD ON GOVT DEBT "
// eststo oxf_oecd: quietly reg av_ryieldifs cov_res_infl_dlcons $avgvars $volvars ///
// 	l.av_govdebt tdum* cdum* , vce(cluster ccode)
// eststo oecd: quietly reg av_ryieldifs cov_res_infl_dlcons $avgvars $volvars ///
// 	l.av_govdebt1 tdum* cdum* , vce(cluster ccode)
// eststo oxf: quietly reg av_ryieldifs cov_res_infl_dlcons $avgvars $volvars ///
// 	l.av_govdebt2 tdum* cdum* , vce(cluster ccode)
// eststo oecd_oxf: quietly reg av_ryieldifs cov_res_infl_dlcons $avgvars $volvars ///
// 	l.av_govdebt3 tdum* cdum* , vce(cluster ccode)
//
// cap drop yield_sample 
// cap: gen yield_sample = e(sample)
// levelsof isocode if yield_sample, local(ccodes) sep(, ) clean
//
// esttab, title("TABLE 9: Sensitivity to debt measures") 					///
// 	drop (*tdum* *cdum* av_res* var_* L.av_* _cons) ar2 se noconstant 	///
// 	compress star(* 0.10 ** 0.05 *** 0.01) obslast replace				///
// 	addnotes("All regressions include country and time fixed effects. Standard errors clustered by country." "Countries: `ccodes'")
// esttab using tables\table_9.csv, 										///
// 	title("Sensitivity to debt measures")  					///
// 	drop (*tdum* *cdum* av_res* var_* L.av_* _cons) ar2 se noconstant 	///
// 	compress star(* 0.10 ** 0.05 *** 0.01) obslast replace				///
// 	addnotes("All regressions include country and time fixed effects. Standard errors clustered by country." "Countries: `ccodes'")
// eststo clear
